
from cuteSV.cuteSV_Description import Generation_VCF_header
from math import log10
import numpy as np
import pysam
import pickle

err = 0.1
prior = float(1/3)
Genotype = ["0/0", "0/1", "1/1"]

def log10sumexp(log10_probs):
    # Normalization of Genotype likelihoods
    m = max(log10_probs)
    return m + log10(sum(pow(10.0, x-m) for x in log10_probs))

def normalize_log10_probs(log10_probs):
    # Adjust the Genotype likelihoods
    log10_probs = np.array(log10_probs)
    lse = log10sumexp(log10_probs)
    return np.minimum(log10_probs - lse, 0.0)

def rescale_read_counts(c0, c1, max_allowed_reads=100):
    """Ensures that n_total <= max_allowed_reads, rescaling if necessary."""
    Total = c0 + c1
    if Total > max_allowed_reads:
        c0 = int(max_allowed_reads * float(c0/Total))
        c1 = max_allowed_reads - c0
    return c0, c1

def cal_GL(c0, c1):
    if c0==3 and c1==1:
        return '0/1', '3,3,24', 3, 3.0
    if c0==6 and c1==2:
        return '0/1', '3,3,45', 3, 3.0
    # Approximate adjustment of events with larger read depth
    c0, c1 = rescale_read_counts(c0, c1) # DR, DV
    # original genotype likelihood
    # ori_GL00 = np.float64(pow((1-err), c0)*pow(err, c1)*comb(c0+c1,c0)*(1-prior)/2)
    # ori_GL11 = np.float64(pow(err, c0)*pow((1-err), c1)*comb(c0+c1,c0)*(1-prior)/2)
    # ori_GL01 = np.float64(pow(0.5, c0+c1)*comb(c0+c1,c0)*prior)
    
    ori_GL00 = np.float64(pow((1-err), c0)*pow(err, c1)*(1-prior)/2)
    ori_GL11 = np.float64(pow(err, c0)*pow((1-err), c1)*(1-prior)/2)
    ori_GL01 = np.float64(pow(0.5, c0+c1)*prior)

    # normalized genotype likelihood
    prob = list(normalize_log10_probs([log10(ori_GL00), log10(ori_GL01), log10(ori_GL11)]))
    GL_P = [pow(10, i) for i in prob]
    PL = [int(np.around(-10*log10(i))) for i in GL_P]
    GQ = [int(-10*log10(GL_P[1] + GL_P[2])), int(-10*log10(GL_P[0] + GL_P[2])), int(-10*log10(GL_P[0] + GL_P[1]))]
    QUAL = abs(np.around(-10*log10(GL_P[0]), 1))

    return Genotype[prob.index(max(prob))], "%d,%d,%d"%(PL[0], PL[1], PL[2]), max(GQ), QUAL

def cal_CIPOS(std, num):
    pos = int(1.96 * std / num ** 0.5)
    return "-%d,%d"%(pos,pos)

def threshold_ref_count(num):
    if num <= 2:
        return 20*num
    elif 3 <= num <= 5:
        return 9*num 
    elif 6 <= num <= 15:
        return 7*num
    else:
        return 5*num

def count_coverage(chr, s, e, f, read_count, up_bound, itround):
    status = 0
    iteration = 0
    primary_num = 0
    for i in f.fetch(chr, s, e):
        iteration += 1
        if i.flag not in [0,16]:
            continue
        primary_num += 1
        if i.reference_start < s and i.reference_end > e:
            read_count.add(i.query_name)
            if len(read_count) >= up_bound:
                status = 1
                break
        if iteration >= itround:
            if float(primary_num/iteration) <= 0.2:
                status = 1
            else:
                status = -1
            break

    return status

def overlap_cover(svs_list, reads_list):
    # [(10024, 12024), (89258, 91258), ...]
    # [[10000, 10468, 0, 'm54238_180901_011437/52298335/ccs'], [10000, 17490, 1, 'm54238_180901_011437/44762027/ccs'], ...]
    sort_list = list()
    idx = 0
    for i in reads_list:
        sort_list.append([i[0], 1, idx, i[2], i[3]])
        sort_list.append([i[1], 2, idx, i[2], i[3]])
        idx += 1
    idx = 0
    for i in svs_list:
        sort_list.append([i[0], 3, idx])
        sort_list.append([i[1], 0, idx])
        idx += 1
    sort_list = sorted(sort_list, key = lambda x:(x[0], x[1]))
    svs_set = set()
    read_set = set()
    overlap_dict = dict()
    cover_dict = dict()
    for node in sort_list:
        if node[1] == 1: # set2(read) left
            read_set.add(node[2])
            for x in svs_set:
                if svs_list[x][1] == node[0]:
                    continue
                if x not in overlap_dict:
                    overlap_dict[x] = set()
                overlap_dict[x].add(node[2])
        elif node[1] == 2: # set2(read) right
            read_set.remove(node[2])
        elif node[1] == 3: # set1(sv) left
            svs_set.add(node[2])
            overlap_dict[node[2]] = set()
            for x in read_set:
                overlap_dict[node[2]].add(x)
            cover_dict[node[2]] = set()
            for x in read_set:
                cover_dict[node[2]].add(x)
        elif node[1] == 0: # set1(sv) right
            svs_set.remove(node[2])
            temp_set = set()
            for x in read_set:
                temp_set.add(x)
            cover_dict[node[2]] = cover_dict[node[2]] & temp_set
    overlap2_dict = dict()
    cover2_dict = dict()
    iteration_dict = dict()
    primary_num_dict = dict()
    for idx in cover_dict:
        iteration_dict[idx] = len(overlap_dict[idx])
        primary_num_dict[idx] = 0
        for x in overlap_dict[idx]:
            if reads_list[x][2] == 1:
                primary_num_dict[idx] += 1
        cover2_dict[idx] = set()
        for x in cover_dict[idx]:
            if reads_list[x][2] == 1:
                cover2_dict[idx].add(reads_list[x][3])
        overlap2_dict[idx] = set()
        for x in overlap_dict[idx]:
            if reads_list[x][2] == 1:
                overlap2_dict[idx].add(reads_list[x][3])
    # duipai(svs_list, reads_list, iteration_dict, primary_num_dict, cover2_dict, overlap2_dict)
    # return iteration_dict, primary_num_dict, cover2_dict
    return iteration_dict, primary_num_dict, cover2_dict, overlap2_dict

def assign_gt(iteration_dict, primary_num_dict, cover_dict, read_id_dict):
    assign_list = list()
    for idx in read_id_dict:
        iteration = iteration_dict[idx]
        primary_num = primary_num_dict[idx]
        read_count = cover_dict[idx]
        DR = 0
        for query in read_count:
            if query not in read_id_dict[idx]:
                DR += 1
        GT, GL, GQ, QUAL = cal_GL(DR, len(read_id_dict[idx]))
        assign_list.append([len(read_id_dict[idx]), DR, GT, GL, GQ, QUAL])
    return assign_list

def assign_gt_fc(iteration_dict, primary_num_dict, cover_dict, overlap_dict, read_id_dict, svtype_id_dict):
    assign_list = list()
    for idx in read_id_dict:
        iteration = iteration_dict[idx]
        primary_num = primary_num_dict[idx]
        if svtype_id_dict[idx] == 'DEL':
            read_count = overlap_dict[idx]
        else:
            read_count = cover_dict[idx]
        DR = 0
        for query in read_count:
            if query not in read_id_dict[idx]:
                DR += 1
        GT, GL, GQ, QUAL = cal_GL(DR, len(read_id_dict[idx]))
        assign_list.append([len(read_id_dict[idx]), DR, GT, GL, GQ, QUAL])
    return assign_list

def duipai(svs_list, reads_list, iteration_dict, primary_num_dict, cover2_dict, overlap2_dict):
    # [(10024, 12024), (89258, 91258), ...]
    # [[10000, 10468, 0, 'm54238_180901_011437/52298335/ccs'], [10000, 17490, 1, 'm54238_180901_011437/44762027/ccs'], ...]
    print('start duipai')
    idx = 0
    correct_cover = 0
    correct_overlap = 0
    bb = set()
    for i in svs_list:
        cover = set()
        overlap = set()
        primary_num = 0
        iteration = 0
        for j in reads_list:
            if (j[0] <= i[0] and j[1] > i[0]) or (i[0] <= j[0] < i[1]):
                iteration += 1
                if j[2] == 1:
                    overlap.add(j[3])
                    primary_num += 1
                    if i[0] >= j[0] and i[1] <= j[1]:
                        cover.add(j[3])
        flag = 0
        if iteration != iteration_dict[idx]:
            print('Iteration error %d:%d(now) %d(ans)'%(idx, iteration_dict[idx], iteration))
        if primary_num != primary_num_dict[idx]:
            print('Primary_num error %d:%d(now) %d(ans)'%(idx, primary_num_dict[idx], primary_num))
        if len(cover) == len(cover2_dict[idx]): flag += 1
        if len(cover - cover2_dict[idx]) == 0: flag += 1
        if len(cover2_dict[idx] - cover) == 0: flag += 1
        if flag != 3:
            print(idx)
            print(cover)
            print(cover2_dict[idx])
            print(cover - cover2_dict[idx])
        else:
            correct_cover += 1
        flag = 0
        if len(overlap) == len(overlap2_dict[idx]): flag += 1
        if len(overlap - overlap2_dict[idx]) == 0: flag += 1
        if len(overlap2_dict[idx] - overlap) == 0: flag += 1
        if flag != 3:
            print(idx)
            print(overlap)
            print(overlap2_dict[idx])
            print(overlap - overlap2_dict[idx])
        else:
            correct_overlap += 1
        idx += 1
    print('Correct iteration cover %d; overlap %d'%(correct_cover, correct_overlap))

def generate_output(args, semi_result, reference, chrom, temporary_dir):
    
    '''
    Generation of VCF format file.
    VCF version: 4.2
    '''

    # genotype_trigger = TriggerGT[args.genotype]

    f=open("%sresults/%s.pickle"%(temporary_dir,chrom), "wb")
    semi_result.sort(key = lambda x:int(x[2]))
    action = args.genotype
    fa_file = pysam.FastaFile(reference)
    try:
        ref_chrom=fa_file.fetch(chrom)
    except:
        raise Exception("No corresponding contig in reference with %s."%(chrom))
    fa_file.close()
    lines=[]
    BATCH_SIZE=1000
    for i in semi_result:
        if i[1] in ["DEL", "INS"]:
            if abs(int(float(i[3]))) > args.max_size and args.max_size != -1:
                continue
            if abs(int(float(i[3]))) < args.min_size:
                continue
            if i[1] == "INS":
                cal_end = int(i[2])
            else:
                cal_end = int(i[2]) + abs(int(float(i[3])))
            info_list = "{PRECISION};SVTYPE={SVTYPE};SVLEN={SVLEN};END={END};CIPOS={CIPOS};CILEN={CILEN};RE={RE};RNAMES={RNAMES}".format(
                PRECISION = "IMPRECISE" if i[8] == "0/0" else "PRECISE", 
                SVTYPE = i[1], 
                SVLEN = i[3], 
                END = str(cal_end), 
                CIPOS = i[5], 
                CILEN = i[6], 
                RE = i[4],
                RNAMES = i[12] if args.report_readid else "NULL")
            if action:
                try:
                    info_list += ";AF=" + str(round(int(i[4]) / (int(i[4]) + int(i[7])), 4))
                except:
                    info_list += ";AF=."
            if i[1] =="DEL":
                info_list += ";STRAND=+-"
            if i[11] == "." or i[11] == None:
                filter_lable = "PASS"
            else:
                filter_lable = "PASS" if float(i[11]) >= 5.0 else "q5"
            lines.append((i[1],"{CHR}\t{POS}\t{ID}\t{REF}\t{ALT}\t{QUAL}\t{PASS}\t{INFO}\t{FORMAT}\t{GT}:{DR}:{RE}:{PL}:{GQ}\n".format(
                CHR = i[0], 
                POS = str(int(i[2])), 
                ID = "cuteSV.%s.<SVID>"%(i[1]),
                REF = ref_chrom[max(int(i[2])-1, 0)] if i[1] == 'INS' else ref_chrom[max(int(i[2])-1, 0):int(i[2])-int(i[3])],
                ALT = "%s"%(ref_chrom[max(int(i[2])-1, 0)]+i[13] if i[1] == 'INS' else ref_chrom[max(int(i[2])-1, 0)]), 
                INFO = info_list, 
                FORMAT = "GT:DR:DV:PL:GQ", 
                GT = i[8],
                DR = i[7],
                RE = i[4],
                PL = i[9],
                GQ = i[10],
                QUAL = i[11],
                PASS = filter_lable)))
        elif i[1] == "DUP":
            if abs(int(float(i[3]))) > args.max_size and args.max_size != -1:
                continue
            cal_end = int(i[2]) + 1 + abs(int(float(i[3])))
            info_list = "{PRECISION};SVTYPE={SVTYPE};SVLEN={SVLEN};END={END};RE={RE};STRAND=-+;RNAMES={RNAMES}".format(
                PRECISION = "IMPRECISE" if i[6] == "0/0" else "PRECISE", 
                SVTYPE = i[1], 
                SVLEN = i[3], 
                END = str(cal_end), 
                RE = i[4],
                RNAMES = i[10] if args.report_readid else "NULL")
            if action:
                try:
                    info_list += ";AF=" + str(round(int(i[4]) / (int(i[4]) + int(i[5])), 4))
                except:
                    info_list += ";AF=."
            if i[9] == ".":
                filter_lable = "PASS"
            else:
                filter_lable = "PASS" if float(i[9]) >= 5.0 else "q5"
            lines.append((i[1],"{CHR}\t{POS}\t{ID}\t{REF}\t{ALT}\t{QUAL}\t{PASS}\t{INFO}\t{FORMAT}\t{GT}:{DR}:{RE}:{PL}:{GQ}\n".format(
                CHR = i[0], 
                POS = str(int(i[2]) + 1), 
                ID = "cuteSV.%s.<SVID>"%(i[1]),
                REF = ref_chrom[int(i[2])],
                ALT = "<%s>"%(i[1]), 
                INFO = info_list, 
                FORMAT = "GT:DR:DV:PL:GQ", 
                GT = i[6],
                DR = i[5],
                RE = i[4],
                PL = i[7],
                GQ = i[8],
                QUAL = i[9],
                PASS = filter_lable)))
        elif i[1] == "INV":
            if abs(int(float(i[3]))) > args.max_size and args.max_size != -1:
                continue
            cal_end = int(i[2]) + 1 + abs(int(float(i[3])))
            info_list = "{PRECISION};SVTYPE={SVTYPE};SVLEN={SVLEN};END={END};RE={RE};STRAND={STRAND};RNAMES={RNAMES}".format(
                PRECISION = "IMPRECISE" if i[6] == "0/0" else "PRECISE", 
                SVTYPE = i[1], 
                SVLEN = i[3], 
                END = str(cal_end), 
                RE = i[4],
                STRAND = i[7],
                RNAMES = i[11] if args.report_readid else "NULL")
            if action:
                try:
                    info_list += ";AF=" + str(round(int(i[4]) / (int(i[4]) + int(i[5])), 4))
                except:
                    info_list += ";AF=."
            if i[10] == ".":
                filter_lable = "PASS"
            else:
                filter_lable = "PASS" if float(i[10]) >= 5.0 else "q5"
            lines.append((i[1],"{CHR}\t{POS}\t{ID}\t{REF}\t{ALT}\t{QUAL}\t{PASS}\t{INFO}\t{FORMAT}\t{GT}:{DR}:{RE}:{PL}:{GQ}\n".format(
                CHR = i[0], 
                POS = str(int(i[2]) + 1), 
                ID = "cuteSV.%s.<SVID>"%(i[1]),
                REF = ref_chrom[int(i[2])],
                ALT = "<%s>"%(i[1]), 
                INFO = info_list, 
                FORMAT = "GT:DR:DV:PL:GQ", 
                GT = i[6],
                DR = i[5],
                RE = i[4],
                PL = i[8],
                GQ = i[9],
                QUAL = i[10],
                PASS = filter_lable)))
        else:
            # BND
            # info_list = "{PRECISION};SVTYPE={SVTYPE};CHR2={CHR2};END={END};RE={RE};RNAMES={RNAMES}".format(
            info_list = "{PRECISION};SVTYPE={SVTYPE};RE={RE};RNAMES={RNAMES}".format(
                PRECISION = "IMPRECISE" if i[7] == "0/0" else "PRECISE", 
                SVTYPE = "BND", 
                # CHR2 = i[3], 
                # END = str(int(i[4]) + 1), 
                RE = i[5],
                RNAMES = i[11] if args.report_readid else "NULL")
            if action:
                try:
                    info_list += ";AF=" + str(round(int(i[5]) / (int(i[5]) + int(i[6])), 4))
                except:
                    info_list += ";AF=."
            if i[10] == ".":
                filter_lable = "PASS"
            else:
                filter_lable = "PASS" if float(i[10]) >= 5.0 else "q5"
            try:
                ref_bnd = ref_chrom[int(i[2])]
            except:
                ref_bnd = 'N'
            lines.append(("BND","{CHR}\t{POS}\t{ID}\t{REF}\t{ALT}\t{QUAL}\t{PASS}\t{INFO}\t{FORMAT}\t{GT}:{DR}:{RE}:{PL}:{GQ}\n".format(
                CHR = i[0], 
                POS = str(int(i[2]) + 1), 
                ID = "cuteSV.%s.<SVID>"%("BND"), 
                REF = ref_bnd,
                ALT = i[1], 
                INFO = info_list, 
                FORMAT = "GT:DR:DV:PL:GQ", 
                GT = i[7],
                DR = i[6],
                RE = i[5],
                PL = i[8],
                GQ = i[9],
                QUAL = i[10],
                PASS = filter_lable)))
        if len(lines)>BATCH_SIZE:
            pickle.dump(lines,f)
            lines=[]
    # with open("%sresults/%s.pickle"%(temporary_dir,chrom), "wb") as f:
    #     pickle.dump(lines,f)
    if len(lines)!=0:
        pickle.dump(lines,f)
    # f.close()
    # return lines


# def generate_pvcf(args, result, contigINFO, argv, ref_g):
def generate_pvcf(args, result, reference, chrom):
    fa_file = pysam.FastaFile(reference)
    try:
        ref_chrom=fa_file.fetch(chrom)
    except:
        raise Exception("No corresponding contig in reference with %s."%(chrom))
    fa_file.close()
    # file = open(args.output, 'w')
    # Generation_VCF_header(file, contigINFO, args.sample, argv)
    # file.write("#CHROM\tPOS\tID\tREF\tALT\tQUAL\tFILTER\tINFO\tFORMAT\t%s\n"%(args.sample))
    # [chrom(0), sv_start, genotype(2), sv_type, sv_end(4), CIPOS, CILEN(6), [gt_re, DR, GT(new), GL, GQ, QUAL], rname(8), svid, ref(10), alts, sv_strand(12), seq]
    lines=[]
    for i in result:
        if i == []:
            continue
        if i[7][5] == "." or i[7][5] == None:
            filter_lable = "PASS"
        else:
            filter_lable = "PASS" if float(i[7][5]) >= 2.5 else "q5"
        if i[3] == 'INS':
            if abs(i[14]) > args.max_size and args.max_size != -1:
                continue
            '''
            if i[11] == '<INS>':
                ref = str(ref_g[i[0]].seq[max(i[1]-1, 0)])
                alt = str(ref_g[i[0]].seq[max(i[1]-1, 0)]) + i[13]
            else:
                ref = i[10]
                alt = i[11]
            '''
            ref = str(ref_chrom[max(i[1]-1, 0)])
            alt = i[11]
            info_list = "{PRECISION};SVTYPE={SVTYPE};SVLEN={SVLEN};END={END};CIPOS={CIPOS};CILEN={CILEN};RE={RE};RNAMES={RNAMES}".format(
                PRECISION = "IMPRECISE" if i[2] == "0/0" else "PRECISE", 
                SVTYPE = i[3], 
                SVLEN = i[14], 
                END = i[1], 
                CIPOS = i[5], 
                CILEN = i[6], 
                RE = i[7][0],
                RNAMES = i[8] if args.report_readid else "NULL")
            try:
                info_list += ";AF=" + str(round(i[7][0] / (i[7][0] + i[7][1]), 4))
            except:
                info_list += ";AF=."
            lines.append("{CHR}\t{POS}\t{ID}\t{REF}\t{ALT}\t{QUAL}\t{PASS}\t{INFO}\t{FORMAT}\t{GT}:{DR}:{RE}:{PL}:{GQ}\n".format(
                CHR = i[0], 
                POS = i[1], 
                ID = i[9],
                REF = ref,
                ALT = alt, 
                QUAL = i[7][5],
                PASS = filter_lable,
                INFO = info_list, 
                FORMAT = "GT:DR:DV:PL:GQ", 
                GT = i[2],
                DR = i[7][1],
                RE = i[7][0],
                PL = i[7][3],
                GQ = i[7][4]
                ))
        elif i[3] == 'DEL':
            if abs(i[14]) > args.max_size and args.max_size != -1:
                continue
            if i[12] == '<DEL>':
                ref = str(ref_chrom[max(int(i[1])-1, 0):int(i[1])-int(i[4])])
                alt = str(ref_chrom[max(int(i[1])-1, 0)])
            else:
                ref = i[10]
                alt = i[11]
            info_list = "{PRECISION};SVTYPE={SVTYPE};SVLEN={SVLEN};END={END};CIPOS={CIPOS};CILEN={CILEN};RE={RE};RNAMES={RNAMES};STRAND=+-".format(
                PRECISION = "IMPRECISE" if i[2] == "0/0" else "PRECISE", 
                SVTYPE = i[3], 
                SVLEN = -abs(i[14]), 
                END = i[1] + abs(i[14]), 
                CIPOS = i[5], 
                CILEN = i[6], 
                RE = i[7][0],
                RNAMES = i[8] if args.report_readid else "NULL")
            try:
                info_list += ";AF=" + str(round(i[7][0] / (i[7][0] + i[7][1]), 4))
            except:
                info_list += ";AF=."
            lines.append("{CHR}\t{POS}\t{ID}\t{REF}\t{ALT}\t{QUAL}\t{PASS}\t{INFO}\t{FORMAT}\t{GT}:{DR}:{RE}:{PL}:{GQ}\n".format(
                CHR = i[0], 
                POS = i[1], 
                ID = i[9],
                REF = ref,
                ALT = alt, 
                QUAL = i[7][5],
                PASS = filter_lable,
                INFO = info_list, 
                FORMAT = "GT:DR:DV:PL:GQ", 
                GT = i[2],
                DR = i[7][1],
                RE = i[7][0],
                PL = i[7][3],
                GQ = i[7][4]
                ))
        elif i[3] == 'DUP':
            if abs(i[4] - i[1]) > args.max_size and args.max_size != -1:
                continue
            info_list = "{PRECISION};SVTYPE={SVTYPE};SVLEN={SVLEN};END={END};RE={RE};RNAMES={RNAMES};STRAND=-+".format(
                PRECISION = "IMPRECISE" if i[2] == "0/0" else "PRECISE", 
                SVTYPE = i[3], 
                SVLEN = abs(i[4] - i[1]), 
                END = i[4], 
                RE = i[7][0],
                RNAMES = i[8] if args.report_readid else "NULL")
            try:
                info_list += ";AF=" + str(round(i[7][0] / (i[7][0] + i[7][1]), 4))
            except:
                info_list += ";AF=."
            lines.append("{CHR}\t{POS}\t{ID}\t{REF}\t{ALT}\t{QUAL}\t{PASS}\t{INFO}\t{FORMAT}\t{GT}:{DR}:{RE}:{PL}:{GQ}\n".format(
                CHR = i[0], 
                POS = i[1], 
                ID = i[9],
                REF = i[10],
                ALT = i[11], 
                QUAL = i[7][5],
                PASS = filter_lable,
                INFO = info_list, 
                FORMAT = "GT:DR:DV:PL:GQ", 
                GT = i[2],
                DR = i[7][1],
                RE = i[7][0],
                PL = i[7][3],
                GQ = i[7][4]
                ))
        elif i[3] == 'INV':
            if abs(i[4] - i[1]) > args.max_size and args.max_size != -1:
                continue
            info_list = "{PRECISION};SVTYPE={SVTYPE};SVLEN={SVLEN};END={END};RE={RE};RNAMES={RNAMES}".format(
                PRECISION = "IMPRECISE" if i[2] == "0/0" else "PRECISE", 
                SVTYPE = i[3], 
                SVLEN = i[4] - i[1], 
                END = i[4], 
                RE = i[7][0],
                RNAMES = i[8] if args.report_readid else "NULL")
            if i[12] != '.':
                info_list += ';STRAND=' + i[12]
            try:
                info_list += ";AF=" + str(round(i[7][0] / (i[7][0] + i[7][1]), 4))
            except:
                info_list += ";AF=."
            lines.append("{CHR}\t{POS}\t{ID}\t{REF}\t{ALT}\t{QUAL}\t{PASS}\t{INFO}\t{FORMAT}\t{GT}:{DR}:{RE}:{PL}:{GQ}\n".format(
                CHR = i[0], 
                POS = i[1], 
                ID = i[9],
                REF = i[10],
                ALT = i[11], 
                QUAL = i[7][5],
                PASS = filter_lable,
                INFO = info_list, 
                FORMAT = "GT:DR:DV:PL:GQ", 
                GT = i[2],
                DR = i[7][1],
                RE = i[7][0],
                PL = i[7][3],
                GQ = i[7][4]
                ))
        else:
            # BND
            info_list = "{PRECISION};SVTYPE={SVTYPE};RE={RE};RNAMES={RNAMES}".format(
                    PRECISION = "IMPRECISE" if i[2] == "0/0" else "PRECISE", 
                    SVTYPE = i[3], 
                    RE = i[7][0],
                    RNAMES = i[8] if args.report_readid else "NULL")
            if i[14] != 0:
                info_list += ';SVLEN=%d'%(i[14])
            try:
                info_list += ";AF=" + str(round(i[7][0] / (i[7][0] + i[7][1]), 4))
            except:
                info_list += ";AF=."
            '''
            if ':' in i[15]:
                info_list += ";CHR2={CHR2};END={END}".format(
                    CHR2 = i[15].split(':')[0],
                    END = i[15].split(':')[1])
            '''
            lines.append("{CHR}\t{POS}\t{ID}\t{REF}\t{ALT}\t{QUAL}\t{PASS}\t{INFO}\t{FORMAT}\t{GT}:{DR}:{RE}:{PL}:{GQ}\n".format(
                CHR = i[0], 
                POS = str(i[1]), 
                ID = i[9],
                REF = i[10],
                ALT = i[11], 
                QUAL = i[7][5],
                PASS = filter_lable,
                INFO = info_list, 
                FORMAT = "GT:DR:DV:PL:GQ", 
                GT = i[2],
                DR = i[7][1],
                RE = i[7][0],
                PL = i[7][3],
                GQ = i[7][4]
                ))
    return lines

            
def load_valuable_chr(path):
    valuable_chr = dict()
    valuable_chr["DEL"] = list()
    valuable_chr["DUP"] = list()
    valuable_chr["INS"] = list()
    valuable_chr["INV"] = list()
    valuable_chr["TRA"] = dict()

    for svtype in ["DEL", "DUP", "INS", "INV"]:
        file = open("%s%s.sigs"%(path, svtype), 'r')
        for line in file:
            chr = line.strip('\n').split('\t')[1]
            if chr not in valuable_chr[svtype]:
                valuable_chr[svtype].append(chr)
        file.close()
        valuable_chr[svtype].sort()

    file = open("%s%s.sigs"%(path, "TRA"), 'r')
    for line in file:
        chr1 = line.strip('\n').split('\t')[1]
        chr2 = line.strip('\n').split('\t')[4]
        
        if chr1 not in valuable_chr["TRA"]:
            valuable_chr["TRA"][chr1] = list()
        if chr2 not in valuable_chr["TRA"][chr1]:
            valuable_chr["TRA"][chr1].append(chr2)

    file.close()
    for chr1 in valuable_chr["TRA"]:
        valuable_chr["TRA"][chr1].sort()

    return valuable_chr

def load_bed(bed_file, Task_list):
    # Task_list: [[chr, start, end], ...]
    bed_regions = dict()
    if bed_file != None:
        # only consider regions in BED file
        with open(bed_file, 'r') as f:
            for line in f:
                seq = line.strip().split('\t')
                if seq[0] not in bed_regions:
                    bed_regions[seq[0]] = list()
                bed_regions[seq[0]].append((int(seq[1]) - 1000, int(seq[2]) + 1000))
        region_list = [[] for i in range(len(Task_list))]
        for chrom in bed_regions:
            bed_regions[chrom].sort(key = lambda x:(x[0], x[1]))
            for item in bed_regions[chrom]:
                for i in range(len(Task_list)):
                    if chrom == Task_list[i][0]:
                        if (Task_list[i][1] <= item[0] and Task_list[i][2] > item[0]) or item[0] <= Task_list[i][1] < item[1]:
                            region_list[i].append(item)
        assert len(region_list) == len(Task_list), "parse bed file error"
        return region_list
    else:
        return None



import numpy as np
import logging
from cuteSV.cuteSV_genotype import cal_GL, threshold_ref_count, count_coverage
import pickle

'''
*******************************************
				TO DO LIST
*******************************************
	1. Adjust format in BND format;
	2. Make TRA calls more sensitivity;
	3. Collect DP info.
	4. Determine (IM)PRECISE type.

	NOTICE:
		For INS larger than read,
		named it with UNRESOLVED.
*******************************************
'''

'''
			*********Description*********
			*	TYPE A:		N[chr:pos[	*
			*	TYPE B:		N]chr:pos]	*
			*	TYPE C:		[chr:pos[N	*
			*	TYPE D:		]chr:pos]N	*
			*****************************
			'''

def resolution_TRA(path, chr_1, read_count, overlap_size, max_cluster_bias, bam_path, action, gt_round, sigs_index):
	if chr_1 not in sigs_index["TRA"].keys():
		return (chr_1,[])
	semi_tra_cluster = list()
	semi_tra_cluster.append([0,0,'','N'])
	candidate_single_SV = list()
	with open("%s%s.pickle"%(path, "TRA"), 'rb') as f:
		f.seek(sigs_index["TRA"][chr_1])
		seqs=pickle.load(f)
	chr_2=seqs[0][2]
	for seq in seqs:
		if seq[2]!=chr_2:
			logging.info("Finished %s-%s:%s."%(chr_1, chr_2, "TRA/BND"))
			if len(semi_tra_cluster) >= read_count:
				if semi_tra_cluster[-1][0] == semi_tra_cluster[-1][1] == 0:
					pass
				else:
					generate_semi_tra_cluster(semi_tra_cluster, 
											chr_1, 
											chr_2, 
											read_count, 
											overlap_size, 
											max_cluster_bias, 
											candidate_single_SV,
											bam_path,
											action,
											gt_round)
			semi_tra_cluster = list()
			semi_tra_cluster.append([0,0,'','N'])
			chr_2=seq[2]
		pos_1 = int(seq[1])
		pos_2 = int(seq[3])
		read_id = seq[4]
		BND_type = seq[0]
	
		if pos_1 - semi_tra_cluster[-1][0] > max_cluster_bias or BND_type != semi_tra_cluster[-1][3]:
			if len(semi_tra_cluster) >= read_count:
				if semi_tra_cluster[-1][0] == semi_tra_cluster[-1][1] == 0:
					pass
				else:
					generate_semi_tra_cluster(semi_tra_cluster, 
											chr_1, 
											chr_2, 
											read_count, 
											overlap_size, 
											max_cluster_bias, 
											candidate_single_SV,
											bam_path,
											action,
											gt_round)
			semi_tra_cluster = []
			semi_tra_cluster.append([pos_1, pos_2, read_id, BND_type])
		else:
			if semi_tra_cluster[-1][0] == semi_tra_cluster[-1][1] == 0:
				semi_tra_cluster = []
				semi_tra_cluster.append([pos_1, pos_2, read_id, BND_type])
			else:
				semi_tra_cluster.append([pos_1, pos_2, read_id, BND_type])

	if len(semi_tra_cluster) >= read_count:
		if semi_tra_cluster[-1][0] == semi_tra_cluster[-1][1] == 0:
			pass
		else:
			generate_semi_tra_cluster(semi_tra_cluster, 
									chr_1, 
									chr_2, 
									read_count, 
									overlap_size, 
									max_cluster_bias, 
									candidate_single_SV,
									bam_path,
									action,
									gt_round)
	logging.info("Finished %s-%s:%s."%(chr_1, chr_2, "TRA/BND"))
	return (chr_1,candidate_single_SV)

def generate_semi_tra_cluster(semi_tra_cluster, chr_1, chr_2, read_count, overlap_size, 
	max_cluster_bias, candidate_single_SV, bam_path, action, gt_round):
	BND_type = semi_tra_cluster[0][3]
	semi_tra_cluster = sorted(semi_tra_cluster, key = lambda x:x[1])
	read_tag = dict()
	temp = list()
	# p1, p2, count
	last_len = semi_tra_cluster[0][1]
	temp.append([semi_tra_cluster[0][0], semi_tra_cluster[0][1], [semi_tra_cluster[0][2]]])
	read_tag[semi_tra_cluster[0][2]] = 0
	for element in semi_tra_cluster:
		if element[1] - last_len > max_cluster_bias:
			temp.append([element[0],element[1],[element[2]]])
			last_len = element[1]
		else:
			temp[-1][0] += element[0]
			temp[-1][1] += element[1]
			temp[-1][2].append(element[2])
			last_len = element[1]

		if element[2] not in read_tag:
			read_tag[element[2]] = 0
	if len(read_tag) < read_count:
		return

	temp = sorted(temp, key = lambda x:-len(set(x[2])))

	if len(temp) > 1 and len(set(temp[1][2])) >= 0.5*read_count:
		if len(set(temp[0][2]))+len(set(temp[1][2])) >= len(semi_tra_cluster)*overlap_size:
			# candidate_single_SV.append("%s\tTRA\t%d\t%s\t%d\t%d\n"%(chr_1, int(temp[0][0]/temp[0][2]), chr_2, int(temp[0][1]/temp[0][2]), len(read_tag)))
			# candidate_single_SV.append("%s\tTRA\t%d\t%s\t%d\t%d\n"%(chr_1, int(temp[1][0]/temp[1][2]), chr_2, int(temp[1][1]/temp[1][2]), len(read_tag)))
			BND_pos_1 = "%s:%s"%(chr_2, int(temp[0][1]/len(temp[0][2])))
			BND_pos_2 = "%s:%s"%(chr_2, int(temp[1][1]/len(temp[1][2])))
			if BND_type == 'A':
				TRA_1 = "N[%s["%(BND_pos_1)
				TRA_2 = "N[%s["%(BND_pos_2)
			elif BND_type == 'B':
				TRA_1 = "N]%s]"%(BND_pos_1)
				TRA_2 = "N]%s]"%(BND_pos_2)
			elif BND_type == 'C':
				TRA_1 = "[%s[N"%(BND_pos_1)
				TRA_2 = "[%s[N"%(BND_pos_2)
			elif BND_type == 'D':
				TRA_1 = "]%s]N"%(BND_pos_1)
				TRA_2 = "]%s]N"%(BND_pos_2)
			else:
				return

			if action:
				import time
				# time_start = time.time()
				DV, DR, GT, GL, GQ, QUAL = call_gt(bam_path, int(temp[0][0]/len(temp[0][2])), 
											int(temp[0][1]/len(temp[0][2])), chr_1, chr_2, set(temp[0][2]), 
											max_cluster_bias, gt_round)
				# cost_time = time.time() - time_start
				# print("BND", chr_1, chr_2, int(temp[0][0]/len(temp[0][2])), int(temp[0][1]/len(temp[0][2])), DR, DV, QUAL, "%.4f"%cost_time)
			else:
				DR = '.'
				GT = './.'
				GL = '.,.,.'
				GQ = "."
				QUAL = "."
			candidate_single_SV.append([chr_1, 
										TRA_1, 
										str(int(temp[0][0]/len(temp[0][2]))), 
										chr_2, 
										str(int(temp[0][1]/len(temp[0][2]))), 
										str(len(set(temp[0][2]))),
										str(DR),
										str(GT),
										str(GL),
										str(GQ),
										str(QUAL),
										str(','.join(set(temp[0][2])))])

			if action:
				import time
				# time_start = time.time()
				DV, DR, GT, GL, GQ, QUAL = call_gt(bam_path, int(temp[1][0]/len(temp[1][2])), 
											int(temp[1][1]/len(temp[1][2])), chr_1, chr_2, set(temp[1][2]), 
											max_cluster_bias, gt_round)
				# cost_time = time.time() - time_start
				# print("BND", chr_1, chr_2, int(temp[1][0]/len(temp[1][2])), int(temp[1][1]/len(temp[1][2])), DR, DV, QUAL, "%.4f"%cost_time)
			else:
				DR = '.'
				GT = './.'
				GL = '.,.,.'
				GQ = "."
				QUAL = "."
			candidate_single_SV.append([chr_1, 
										TRA_2, 
										str(int(temp[1][0]/len(temp[1][2]))), 
										chr_2, 
										str(int(temp[1][1]/len(temp[1][2]))), 
										str(len(set(temp[1][2]))),
										str(DR),
										str(GT),
										str(GL),
										str(GQ),
										str(QUAL),
										str(','.join(set(temp[1][2])))])
	else:
		if len(set(temp[0][2])) >= len(semi_tra_cluster)*overlap_size:
			# print("%s\tTRA\t%d\t%s\t%d\t%d"%(chr_1, int(temp[0][0]/temp[0][2]), chr_2, int(temp[0][1]/temp[0][2]), len(read_tag)))
			# candidate_single_SV.append("%s\tTRA\t%d\t%s\t%d\t%d\n"%(chr_1, int(temp[0][0]/temp[0][2]), chr_2, int(temp[0][1]/temp[0][2]), len(read_tag)))
			BND_pos = "%s:%s"%(chr_2, int(temp[0][1]/len(temp[0][2])))
			if BND_type == 'A':
				TRA = "N[%s["%(BND_pos)
			elif BND_type == 'B':
				TRA = "N]%s]"%(BND_pos)
			elif BND_type == 'C':
				TRA = "[%s[N"%(BND_pos)
			elif BND_type == 'D':
				TRA = "]%s]N"%(BND_pos)
			else:
				return

			if action:
				import time
				# time_start = time.time()
				DV, DR, GT, GL, GQ, QUAL = call_gt(bam_path, int(temp[0][0]/len(temp[0][2])), 
											int(temp[0][1]/len(temp[0][2])), chr_1, chr_2, set(temp[0][2]), 
											max_cluster_bias, gt_round)
				# cost_time = time.time() - time_start
				# print("BND", chr_1, chr_2, int(temp[0][0]/len(temp[0][2])), int(temp[0][1]/len(temp[0][2])), DR, DV, QUAL, "%.4f"%cost_time)
			else:
				DR = '.'
				GT = './.'
				GL = '.,.,.'
				GQ = "."
				QUAL = "."
			candidate_single_SV.append([chr_1, 
										TRA, 
										str(int(temp[0][0]/len(temp[0][2]))), 
										chr_2, 
										str(int(temp[0][1]/len(temp[0][2]))), 
										str(len(set(temp[0][2]))),
										str(DR),
										str(GT),
										str(GL),
										str(GQ),
										str(QUAL),
										str(','.join(set(temp[0][2])))])


def run_tra(args):
	return resolution_TRA(*args)

def call_gt(bam_path, pos_1, pos_2, chr_1, chr_2, read_id_list, max_cluster_bias, gt_round):
	import pysam
	bamfile = pysam.AlignmentFile(bam_path)
	querydata = set()
	search_start = max(int(pos_1) - max_cluster_bias, 0)
	search_end = min(int(pos_1) + max_cluster_bias, bamfile.get_reference_length(chr_1))

	up_bound = threshold_ref_count(len(read_id_list))

	status = count_coverage(chr_1, 
							search_start, 
							search_end, 
							bamfile, 
							querydata, 
							up_bound, 
							gt_round)

	if status == -1:
		DR = '.'
		GT = "./."
		GL = ".,.,."
		GQ = "."
		QUAL = "."

	elif status == 1:
		DR = 0
		for query in querydata:
			if query not in read_id_list:
				DR += 1
		GT, GL, GQ, QUAL = cal_GL(DR, len(read_id_list))

	else:
		search_start = max(int(pos_2) - max_cluster_bias, 0)
		search_end = min(int(pos_2) + max_cluster_bias, bamfile.get_reference_length(chr_2))
		status_2 = count_coverage(chr_2, 
									search_start, 
									search_end, 
									bamfile, 
									querydata, 
									up_bound, 
									gt_round)
		# status_2 judgement
		DR = 0
		for query in querydata:
			if query not in read_id_list:
				DR += 1
		GT, GL, GQ, QUAL = cal_GL(DR, len(read_id_list))

	bamfile.close()
	return len(read_id_list), DR, GT, GL, GQ, QUAL



import sys
import argparse
import logging
import time

callset = {1: "cuteSV", 2: "Sniflles", 3: "PBSV", 4: "SVIM"}

USAGE="""\
	Evaluate SV callset on NA19240 dataset
"""

def parseArgs(argv):
	parser = argparse.ArgumentParser(prog="NA19240_eval", description=USAGE, formatter_class=argparse.RawDescriptionHelpFormatter)
	parser.add_argument("base", type=str, help="Base vcf file of NA19240.")
	parser.add_argument("cuteSV", type=str, help="CuteSV vcf file of NA19240.")
	parser.add_argument("sniffles", type=str, help="Sniffles vcf file of NA19240.")
	parser.add_argument("pbsv", type=str, help="PBSV vcf file of NA19240.")
	parser.add_argument("svim", type=str, help="SVIM vcf file of NA19240.")
	parser.add_argument('-b', '--bias', help = "Bias of overlaping.[%(default)s]", default = 0.7, type = float)
	parser.add_argument('-o', '--offect', help = "Offect of breakpoint distance.[%(default)s]", default = 1000, type = int)
	args = parser.parse_args(argv)
	return args

def pase_base_info(seq):
	info = {'SVLEN': 0, 'END': 0, "SVTYPE": '', "RE": 0}
	for i in seq.split(';'):
		if i.split('=')[0] in ["SVLEN", "END", "RE"]:
			try:
				info[i.split('=')[0]] = abs(int(i.split('=')[1]))
			except:
				pass
		if i.split('=')[0] == "SVTYPE":
			info[i.split('=')[0]] = i.split('=')[1][0:3]
	return info


def load_base(base_path):
	base_call = dict()
	file = open(base_path, 'r')
	for line in file:
		seq = line.strip('\n').split("\t")
		if seq[0][0] == '#':
			continue

		chr = seq[0]
		pos = int(seq[1])
		ALT = seq[4][1:4]
		if ALT not in ["INS", "INV", "DEL", "DUP"]:
			continue
		if ALT == "DUP":
			ALT = "INS"
		info = pase_base_info(seq[7])
		if ALT not in base_call:
			base_call[ALT] = dict()

		if chr not in base_call[ALT]:
			base_call[ALT][chr] = list()

		if ALT == "INV":
			base_call[ALT][chr].append([pos, info["END"] - pos + 1, info["END"], 0])
		else:
			if info["SVLEN"] >= 50 and info["SVLEN"] <= 100000:
				base_call[ALT][chr].append([pos, info["SVLEN"], info["END"], 0])
	file.close()
	return base_call

def load_cuteSV(cuteSV_path):
	# inv_tag = 0
	last_inv = list()
	cuteSV_call = dict()
	file = open(cuteSV_path, 'r')
	for line in file:
		seq = line.strip('\n').split("\t")
		if seq[0][0] == '#':
			continue

		chr = seq[0]
		pos = int(seq[1])
		ALT = seq[2][7:10]
		# if ALT == "DUP":
		# 	ALT = "INS"
		if ALT not in ["INS", "INV", "DEL", "DUP"]:
			continue

		info = pase_base_info(seq[7])
		if ALT not in cuteSV_call:
			cuteSV_call[ALT] = dict()

		if chr not in cuteSV_call[ALT]:
			cuteSV_call[ALT][chr] = list()

		if info["SVLEN"] >= 50 and info["SVLEN"] <= 100000:
			if ALT == "INV":
				last_inv.append([ALT, chr, pos, info["SVLEN"], info["END"], info["RE"]])
				# if inv_tag == 0
			else:
				cuteSV_call[ALT][chr].append([pos, info["SVLEN"], info["END"], 0])
				# inv_tag = 0
				if len(last_inv):
					last_inv = sorted(last_inv, key = lambda x:-x[3])
					cuteSV_call[last_inv[0][0]][last_inv[0][1]].append([last_inv[0][2], last_inv[0][3], last_inv[0][4], 0])
					last_inv = list()
	file.close()
	return cuteSV_call

def load_sniffles(sniffles_path):
	sniffles_call = dict()
	last_inv = list()
	file = open(sniffles_path, 'r')
	for line in file:
		seq = line.strip('\n').split("\t")
		if seq[0][0] == '#':
			continue

		chr = seq[0]
		pos = int(seq[1])
		info = pase_base_info(seq[7])

		if info["SVTYPE"] not in ["INS", "INV", "DEL", "DUP"]:
			continue

		# if info["SVTYPE"] == "DUP":
		# 	info["SVTYPE"] = "INS"

		if info["SVTYPE"] not in sniffles_call:
			sniffles_call[info["SVTYPE"]] = dict()

		if chr not in sniffles_call[info["SVTYPE"]]:
			sniffles_call[info["SVTYPE"]][chr] = list()

		if info["SVLEN"] >= 50 and info["SVLEN"] <= 100000:
			if info["SVTYPE"] == "INV":
				last_inv.append([info["SVTYPE"], chr, pos, info["SVLEN"], info["END"], info["RE"]])
			else:
				sniffles_call[info["SVTYPE"]][chr].append([pos, info["SVLEN"], info["END"], 0])
				if len(last_inv):
					last_inv = sorted(last_inv, key = lambda x:-x[3])
					sniffles_call[last_inv[0][0]][last_inv[0][1]].append([last_inv[0][2], last_inv[0][3], last_inv[0][4], 0])
					last_inv = list()

	file.close()
	return sniffles_call

def load_pbsv(pbsv_path):
	pbsv_call = dict()
	file = open(pbsv_path, 'r')
	for line in file:
		seq = line.strip('\n').split("\t")
		if seq[0][0] == '#':
			continue

		chr = seq[0]
		pos = int(seq[1])

		info = pase_base_info(seq[7])

		if info["SVTYPE"] not in ["INS", "INV", "DEL", "DUP"]:
			continue

		# if info["SVTYPE"] == "DUP":
		# 	info["SVTYPE"] = "INS"

		if info["SVTYPE"] not in pbsv_call:
			pbsv_call[info["SVTYPE"]] = dict()

		if chr not in pbsv_call[info["SVTYPE"]]:
			pbsv_call[info["SVTYPE"]][chr] = list()

		if info["SVTYPE"] == "INV":
			pbsv_call[info["SVTYPE"]][chr].append([pos, info["END"] - pos + 1, info["END"], 0])
		else:
			if info["SVLEN"] >= 50 and info["SVLEN"] <= 100000:
				pbsv_call[info["SVTYPE"]][chr].append([pos, info["SVLEN"], info["END"], 0])
	file.close()
	return pbsv_call

def load_svim(base_path):
	base_call = dict()
	file = open(base_path, 'r')
	for line in file:
		seq = line.strip('\n').split("\t")
		if seq[0][0] == '#':
			continue

		chr = seq[0]
		pos = int(seq[1])
		ALT = seq[4][1:4]
		if ALT not in ["INS", "INV", "DEL", "DUP"]:
			continue
		# if ALT == "DUP":
		# 	ALT = "INS"
		info = pase_base_info(seq[7])
		if ALT not in base_call:
			base_call[ALT] = dict()

		if chr not in base_call[ALT]:
			base_call[ALT][chr] = list()

		if ALT == "INV":
			base_call[ALT][chr].append([pos, info["END"] - pos + 1, info["END"], 0])
		else:
			if info["SVLEN"] >= 50 and info["SVLEN"] <= 100000:
				base_call[ALT][chr].append([pos, info["SVLEN"], info["END"], 0])
	file.close()
	return base_call

def cmp_callsets(base, call, flag, Bias, Offect):
	for svtype in base:
		if svtype not in call:
			continue
		else:
			for chr in base[svtype]:
				if chr not in call[svtype]:
					continue
				else:
					for i in base[svtype][chr]:
						for j in call[svtype][chr]:
							if i[0] - Offect <= j[0] <= i[2] + Offect or i[0] - Offect <= j[2] <= i[2] + Offect or j[0] - Offect <= i[0] <= j[2] + Offect:
								if min(i[1], j[1])*1.0/max(i[1], j[1]) >= Bias:
									i[3] = flag
									j[3] = flag
								else:
									pass
	total_base = 0
	tp_base = 0
	# for svtype in ["INS"]:
	# for svtype in ["DUP"]:
	# for svtype in ["DEL"]:
	# for svtype in ["INS", "DEL"]:
	for svtype in ["INS", "DEL", "INV"]:
	# for svtype in ["INS", "DEL", "INV", "DUP"]:
	# for svtype in ["INV"]:
		for chr in base[svtype]:
			for i in base[svtype][chr]:
				total_base += 1
				if i[3] == flag:
					tp_base += 1
				# else:
				# 	print(flag, svtype, chr, i[0], i[1], i[2])
	# logging.info("Base count: %d"%(total_base))
	# logging.info("TP-base count: %d"%(tp_base))
	logging.info("====%s===="%(callset[flag]))
	total_call = 0
	tp_call = 0
	# for svtype in ["INS"]:
	# for svtype in ["DUP"]:
	# for svtype in ["DEL"]:
	# for svtype in ["INS", "DEL"]:
	for svtype in ["INS", "DEL", "INV"]:
	# for svtype in ["INS", "DEL", "INV", "DUP"]:
	# for svtype in ["INV"]:
		for chr in call[svtype]:
			for i in call[svtype][chr]:
				total_call += 1
				if i[3] == flag:
					tp_call += 1

	
	logging.info("Camp count: %d"%(total_call))
	logging.info("TP-call count: %d"%(tp_call))
	logging.info("Precision: %.2f"%(100.0*tp_call/total_call))
	logging.info("Recall: %.2f"%(100.0*tp_base/total_base))
	logging.info("F-measure: %.2f"%(200.0*tp_base*tp_call/(total_base*tp_call+tp_base*total_call)))



def main_ctrl(args):
	# pass
	base_call = load_base(args.base)
	cuteSV_call = load_cuteSV(args.cuteSV)
	sniffles_call = load_sniffles(args.sniffles)
	pbsv_call = load_pbsv(args.pbsv)
	svim_call = load_svim(args.svim)
	# for svtype in sniffles_call:
	# 	for chr in sniffles_call[svtype]:
	# 		for i in sniffles_call[svtype][chr]:
	# 			print(svtype, chr, i)

	cmp_callsets(base_call, cuteSV_call, 1, args.bias, args.offect)
	cmp_callsets(base_call, sniffles_call, 2, args.bias, args.offect)
	cmp_callsets(base_call, pbsv_call, 3, args.bias, args.offect)
	cmp_callsets(base_call, svim_call, 4, args.bias, args.offect)
	

def main(argv):
	args = parseArgs(argv)
	setupLogging(False)
	# print args
	starttime = time.time()
	main_ctrl(args)
	logging.info("Finished in %0.2f seconds."%(time.time() - starttime))

def setupLogging(debug=False):
	logLevel = logging.DEBUG if debug else logging.INFO
	logFormat = "%(asctime)s [%(levelname)s] %(message)s"
	logging.basicConfig( stream=sys.stderr, level=logLevel, format=logFormat )
	logging.info("Running %s" % " ".join(sys.argv))

if __name__ == '__main__':
	main(sys.argv[1:])



import sys
import argparse
import logging
import time
import vcf

def phase_bnd(alt):
	if alt[0] in [']', '[']:
		chr2 = alt.split(':')[0][1:]
		pos2 = int(alt.split(':')[1][:-2])
	else:
		chr2 = alt.split(':')[0][2:]
		pos2 = int(alt.split(':')[1][:-1])

	return chr2, pos2

def main_ctrl(args):
	filein = vcf.Reader(open(args.vcf, 'r'))
	fileout = open(args.bedpe, 'w')

	fileout.write("#chrom1\tstart1\tend1\tchrom2\tstart2\tend2\tname\tscore\tstarnd1\tstrand2\tsvtype\tnumber_of_support_read\n")

	for record in filein:
		if record.INFO['SVTYPE'] in ["DEL", "INS", "INV", "DUP"]:
			fileout.write("{chrom1}\t{start1}\t{end1}\t{chrom2}\t{start2}\t{end2}\t{name}\t{score}\t{starnd1}\t{strand2}\t{svtype}\t{RE}\n".format(
				chrom1 = record.CHROM,
				start1 = record.POS+1,
				end1 = record.POS+1,
				chrom2 = record.CHROM,
				start2 = record.INFO["END"]+1,
				end2 = record.INFO["END"]+1,
				name = record.ID,
				score = record.QUAL,
				starnd1 = '+',
				strand2 = '-',
				svtype = record.INFO['SVTYPE'],
				RE = record.INFO["RE"]
				))

		else:
			chr2, pos2 = phase_bnd(str(record.ALT[0]))
			fileout.write("{chrom1}\t{start1}\t{end1}\t{chrom2}\t{start2}\t{end2}\t{name}\t{score}\t{starnd1}\t{strand2}\t{svtype}\t{RE}\n".format(
				chrom1 = record.CHROM,
				start1 = record.POS+1,
				end1 = record.POS+1,
				chrom2 = chr2,
				start2 = pos2+1,
				end2 = pos2+1,
				name = record.ID,
				score = record.QUAL,
				starnd1 = '+',
				strand2 = '-',
				svtype = record.INFO['SVTYPE'],
				RE = record.INFO["RE"]
				))
	fileout.close()

def main(argv):
	args = parseArgs(argv)
	setupLogging(False)
	starttime = time.time()
	main_ctrl(args)
	logging.info("Finished in %0.2f seconds."%(time.time() - starttime))

USAGE="""\
	Convert vcf file generated from cuteSV to bedpe file.
	Author: Tao Jiang
	Email: tjiang@hit.edu.cn
"""

def parseArgs(argv):
	parser = argparse.ArgumentParser(prog="vcf2bedpe.py", description=USAGE, formatter_class=argparse.RawDescriptionHelpFormatter)
	parser.add_argument("vcf", type=str, help="A vcf file generated from cuteSV.")
	parser.add_argument("bedpe", type=str, help="The output bedpe file name.")
	args = parser.parse_args(argv)
	return args

def setupLogging(debug=False):
	logLevel = logging.DEBUG if debug else logging.INFO
	logFormat = "%(asctime)s [%(levelname)s] %(message)s"
	logging.basicConfig( stream=sys.stderr, level=logLevel, format=logFormat )
	logging.info("Running %s" % " ".join(sys.argv))

if __name__ == '__main__':
	main(sys.argv[1:])




import sys
import argparse
import logging
import time

def pase_info(seq):
	info = {'SVLEN': 0, 'END': 0, "SVTYPE": '', "RE": 0, "CHR2": ''}
	for i in seq.split(';'):
		if i.split('=')[0] in ["SVLEN", "END", "RE"]:
			try:
				info[i.split('=')[0]] = abs(int(float(i.split('=')[1])))
			except:
				pass
		if i.split('=')[0] in ["CHR2"]:
			info[i.split('=')[0]] = i.split('=')[1]
		if i.split('=')[0] in ["SVTYPE"]:
			info[i.split('=')[0]] = i.split('=')[1][0:3]

	return info

def phase_GT(seq):
	i = seq.split(':')
	if i[0] in ['0/1', '1/0']:
		return 'het'
	elif i[0] == '1/1':
		return 'hom'
	else:
		return 'unknown'

def load_callset(path):
	callset = dict()
	file = open(path, 'r')
	for line in file:
		seq = line.strip('\n').split('\t')
		if seq[0][0] == '#':
			continue

		chr = seq[0]
		pos = int(seq[1])
		info = pase_info(seq[7])

		if info['SVTYPE'] in ['DEL', 'INS', 'DUP', 'INV']:
			if info['SVTYPE'] not in callset:
				callset[info['SVTYPE']] = list()
			if info['SVLEN'] == 0:
				info['SVLEN'] = info['END'] - pos + 1

			callset[info['SVTYPE']].append([chr, pos, info['END'], info['SVLEN'], phase_GT(seq[9]), 0])

		if info['SVTYPE'] == "BND":
			if seq[4][0] == ']':
				form = ']]N'
				chr2 = seq[4].split(':')[0][1:]
				pos2 = int(seq[4].split(':')[1][:-2])
			elif seq[4][0] == '[':
				form = '[[N'
				chr2 = seq[4].split(':')[0][1:]
				pos2 = int(seq[4].split(':')[1][:-2])
			else:
				if seq[4][1] == ']':
					form = 'N]]'
					chr2 = seq[4].split(':')[0][2:]
					pos2 = int(seq[4].split(':')[1][:-1])
				else:
					form = 'N[['
					chr2 = seq[4].split(':')[0][2:]
					pos2 = int(seq[4].split(':')[1][:-1])
			if info['SVTYPE'] not in callset:
				callset[info['SVTYPE']] = list()
			if info['END'] == 0:
				info['CHR2'] = chr2
				info['END'] = pos2

			callset[info['SVTYPE']].append([chr, pos, info['CHR2'], info['END'], form, phase_GT(seq[9]), 0])

	file.close()
	return callset

def eva_record(call_A, call_B, bias, offect, gt):
	# call_A 0/1
	# call_B 1/1
	for svtype in call_A:
		if svtype not in call_B:
			continue
		else:
			for i in call_B[svtype]:
				if i[-2] not in gt:
					continue
				for j in call_A[svtype]:
					if i[0] != j[0]:
						continue
					if svtype == 'INS':
						if abs(i[1]-j[1]) <= offect and float(min(i[3], j[3])/max(i[3], j[3])) >= bias:
							i[-1] = 1
					elif svtype == 'BND':
						if i[2] == j[2] and i[4] == j[4]:
							# chromosome + strand
							if abs(i[1] - j[1]) <= offect and abs(i[3] - j[3]) <= offect:
								i[-1] = 1
					else:
						if max(i[1]-offect, j[1]) <= min(i[2]+offect, j[2]) and float(min(i[3],j[3])/max(i[3],j[3])) >= bias:
							# callset[info['SVTYPE']].append([chr, pos, info['END'], info['SVLEN'], phase_GT(seq[9]), 0])
							i[-1] = 1

def statistics_true_possitive(callset, SVTYPE, gt):
	record = 0
	true_record = 0
	if SVTYPE == "ALL":
		for svtype in callset:
			for i in callset[svtype]:
				if i[-2] in gt:
					record += 1
					if i[-1] == 1:
						true_record += 1
	else:
		if SVTYPE not in callset:
			return record, true_record
		for i in callset[SVTYPE]:
			if i[-2] in gt:
				record += 1
				if i[-1] == 1:
					true_record += 1

	return record, true_record


def main_ctrl(args):	
	logging.info("Load SV callset of selected caller.")

	call_child = load_callset(args.F1)
	call_father = load_callset(args.MP)
	call_mother = load_callset(args.FP)

	logging.info("Evaluate accuracy and sensitivity.")
	eva_record(call_child, call_father, args.bias, args.offect, ['hom'])
	eva_record(call_child, call_mother, args.bias, args.offect, ['hom'])
	eva_record(call_father, call_child, args.bias, args.offect, ['hom', 'het'])
	eva_record(call_mother, call_child, args.bias, args.offect, ['hom', 'het'])
	svtype = ["DEL", "INS", "INV", "BND", "DUP", 'ALL']
	for i in svtype:
		record, true_record = statistics_true_possitive(call_child, i, ['hom', 'het'])
		logging.info("%s-%s: %d\t%d\t%.2f."%('F1', i, record, true_record, 100*float(true_record/record)))
		record, true_record = statistics_true_possitive(call_father, i, ['hom'])
		logging.info("%s-%s: %d\t%d\t%.2f."%('MP', i, record, true_record, 100*float(true_record/record)))
		record, true_record = statistics_true_possitive(call_mother, i, ['hom'])
		logging.info("%s-%s: %d\t%d\t%.2f."%('FP', i, record, true_record, 100*float(true_record/record)))


def main(argv):
	args = parseArgs(argv)
	setupLogging(False)
	# print args
	starttime = time.time()
	main_ctrl(args)
	logging.info("Finished in %0.2f seconds."%(time.time() - starttime))

USAGE="""\
	Evaluate SV callset generated by cuteSV
"""

def parseArgs(argv):
	parser = argparse.ArgumentParser(prog="Trio_eval", description=USAGE, formatter_class=argparse.RawDescriptionHelpFormatter)
	# parser.add_argument("CALLER", type=str, help="Choose a caller")
	parser.add_argument("MP", type=str, help="Male parent callsets")
	parser.add_argument('FP', type=str, help = "Female parent callsets")
	parser.add_argument('F1', type=str, help = "Offspring callsets")
	# parser.add_argument("MP_c", type=int, help="Coverage of male parent callsets")
	# parser.add_argument('FP_c', type=int, help = "Coverage of female parent callsets")
	# parser.add_argument('F1_c', type=int, help = "Coverage of offspring callsets")
	parser.add_argument('-b', '--bias', help = "Bias of overlaping.[%(default)s]", default = 0.7, type = float)
	parser.add_argument('-o', '--offect', help = "Offect of translocation overlaping.[%(default)s]", default = 1000, type = int)
	args = parser.parse_args(argv)
	return args

def setupLogging(debug=False):
	logLevel = logging.DEBUG if debug else logging.INFO
	logFormat = "%(asctime)s [%(levelname)s] %(message)s"
	logging.basicConfig( stream=sys.stderr, level=logLevel, format=logFormat )
	logging.info("Running %s" % " ".join(sys.argv))

if __name__ == '__main__':
	main(sys.argv[1:])



from string import Template
import tempfile
import subprocess, signal, logging, os, stat, sys

class Alarm(Exception):
    pass

def alarm_handler(signum, frame):
    raise Alarm
    
def setupLogging(debug=False):
    logLevel = logging.DEBUG if debug else logging.INFO
    logFormat = "%(asctime)s [%(levelname)s] %(message)s"
    logging.basicConfig( stream=sys.stderr, level=logLevel, format=logFormat )
    logging.info("Running %s" % " ".join(sys.argv))

def exe(cmd, timeout=-1):
    """
    Executes a command through the shell.
    timeout in minutes! so 1440 mean is 24 hours.
    -1 means never
    """
    proc = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, \
                            stderr=subprocess.STDOUT, close_fds=True,\
                            preexec_fn=os.setsid)
    signal.signal(signal.SIGALRM, alarm_handler)
    if timeout > 0:
        signal.alarm(int(timeout*60))  
    try:
        stdoutVal, stderrVal =  proc.communicate()
        signal.alarm(0)  # reset the alarm
    except Alarm:
        logging.error(("Command was taking too long. "
                       "Automatic Timeout Initiated after %d" % (timeout)))
        os.killpg(proc.pid, signal.SIGTERM)
        proc.kill()
        return 214,None,None
    
    retCode = proc.returncode
    return retCode,stdoutVal,stderrVal

class Command():
    def __init__(self, cmd, jobname, stdout, stderr):
        self.cmd = cmd
        self.jobname = jobname
        self.stdout = stdout
        self.stderr = stderr
    
    def asDict(self):
        return {"CMD":self.cmd, "JOBNAME":self.jobname, \
                "STDOUT":self.stdout, "STDERR":self.stderr}
    
class CommandRunner():
    """
    Uses a command template to run stuff. This is helpful for cluster commands
    and chunking several commands together
    """
    def __init__(self, template=None, njobs=0):
        """
        template: a string that will become the template for submitting to your cluster:
            #you can also go ahead and specify a string.Template
            default is to not submit to your cluster
            ${CMD} > ${STDOUT} 2> ${STDERR}
        njobs: (0)
            for clumping commands together and submitting them in a script
        """
        if template is None:
            template = "${CMD} > ${STDOUT} 2> ${STDERR}"
            self.runType = "Running"
        else:
            self.runType = "Submitting"
        self.template = Template(template)
        self.njobs = njobs
    
    def __call__(self, cmds, wDir = None, id = None):
        """
        Executes Commands - can either be a list or a single Command
        wDir is the working directory where chunk scripts will be written
        if id is None a random identifier will be applied when chunking
        """
        if wDir is None:
            wDir = "./"
        
        if type(cmds) != list:
            cmd = self.buildCommand(cmds)
            return exe(cmd)
        
        if self.njobs == 0:
            outRet = []
            for c in cmds:
                outRet.append(exe(self.buildCommand(c)))
            return outRet
        
        if id is None:
            id = tempfile.mkstemp(dir=wDir)[1]
        
        outputRet =[]
        for chunk, commands in enumerate( partition(cmds, self.njobs) ):
            outScript = open(os.path.join(wDir, "%s_chunk%d.sh" % (id, chunk)),'w')
            outScript.write("#!/bin/bash\n\n")
            for c in commands:
                outScript.write(c.cmd+"\n")
            outScript.close()
            #Add executeable 
            existing_permissions = stat.S_IMODE(os.stat(outScript.name).st_mode)
            if not os.access(outScript.name, os.X_OK):
                new_permissions = existing_permissions | stat.S_IXUSR
                os.chmod(outScript.name, new_permissions)
                
            submit = Command(outScript.name, \
                            id + "_chunk%d" % chunk, \
                            os.path.join(wDir, id + ("_chunk%d.out" % chunk)), \
                            os.path.join(wDir, id + ("_chunk%d.err" % chunk)))
            cmd = self.buildCommand(submit)
            outputRet.append(exe(cmd))
            
        return outputRet
        
    def checkTemplate(self):
        """
        Checks that my template works okay
        """
        temp.update({"CMD":"test", \
                     "STDOUT":"testo", \
                     "STDERR":"teste", \
                     "JOBNAME":"testn"})
        try:
            w = self.template.substitute(temp)
        except KeyError:
            logging.error("Your submission template is invalid ")
            sys.exit(1)

    def buildCommand(self, cmdSetup):
        """
        substitutes a template with a Command
        """
        return self.template.substitute(cmdSetup.asDict())

def partition(n,m):
    """
    Helper function. splits list n into m partitions
    """
    p = map(lambda x: list(), range(m))
    index = 0
    for item in n:
        p[index].append(item)
        if index < m-1:
            index += 1
        else:
            index = 0
    return filter(lambda x: len(x)>0, p)



import sys
import argparse
import logging
import time

def pase_info(seq):
    info = {'SVLEN': 0, 'END': 0, "SVTYPE": '', "RE": 0, "CHR2": ''}
    for i in seq.split(';'):
        if i.split('=')[0] in ["SVLEN", "END", "RE"]:
            try:
                info[i.split('=')[0]] = abs(int(float(i.split('=')[1])))
            except:
                pass
        if i.split('=')[0] in ["CHR2"]:
            info[i.split('=')[0]] = i.split('=')[1]
        if i.split('=')[0] in ["SVTYPE"]:
            info[i.split('=')[0]] = i.split('=')[1][0:3]

    return info

def phase_GT(seq):
    i = seq.split(':')
    if i[0] in ['0/1', '1/0']:
        return 'het'
    elif i[0] == '1/1':
        return 'hom'
    else:
        return 'unknown'

def load_callset(path):
    callset = dict()
    file = open(path, 'r')
    for line in file:
        seq = line.strip('\n').split('\t')
        if seq[0][0] == '#':
            continue

        chr = seq[0]
        pos = int(seq[1])
        info = pase_info(seq[7])
        if info['SVTYPE'] == "TRA":
            info['SVTYPE'] = "BND"

        if info['SVTYPE'] == "BND":
            if seq[4][0] == ']':
                form = ']]N'
                chr2 = seq[4].split(':')[0][1:]
                pos2 = int(seq[4].split(':')[1][:-2])
            elif seq[4][0] == '[':
                form = '[[N'
                chr2 = seq[4].split(':')[0][1:]
                pos2 = int(seq[4].split(':')[1][:-2])
            else:
                if seq[4][1] == ']':
                    form = 'N]]'
                    chr2 = seq[4].split(':')[0][2:]
                    pos2 = int(seq[4].split(':')[1][:-1])
                else:
                    form = 'N[['
                    chr2 = seq[4].split(':')[0][2:]
                    pos2 = int(seq[4].split(':')[1][:-1])
            if info['SVTYPE'] not in callset:
                callset[info['SVTYPE']] = list()
            if info['END'] == 0:
                info['CHR2'] = chr2
                info['END'] = pos2
            try:
                if int(chr) <= int(info['CHR2']):
                    if form == 'N[[':
                        form = ']]N'
                    if form == ']]N':
                        form = 'N[['
                    callset[info['SVTYPE']].append([chr, pos, info['CHR2'], info['END'], form, phase_GT(seq[9]), 0])
                else:
                    callset[info['SVTYPE']].append([info['CHR2'], info['END'], chr, pos, form, phase_GT(seq[9]), 0])
            except:
                callset[info['SVTYPE']].append([chr, pos, info['CHR2'], info['END'], form, phase_GT(seq[9]), 0])

    file.close()
    return callset

def eval(call, ans, offect):
    tpcall = 0
    for i in call["BND"]:
        for j in ans["BND"]:
            if i[0] == j[0] and i[2] == j[2] and abs(i[1]-j[1]) <= offect and abs(i[3]-j[3]) <= offect:
                tpcall += 1
                break
    fp = len(call["BND"]) - tpcall
    return fp, len(call["BND"])

def main_ctrl(args):
    # load ground truth set
    base_list = load_callset(args.base)
    comp_list = load_callset(args.comp)
    
    fp, total = eval(comp_list, base_list, args.offect)
    logging.info('False positive in BND: %d'%(fp))
    logging.info('Total amount of BND: %d'%(total))

def main(argv):
	args = parseArgs(argv)
	setupLogging(False)
	# print args
	starttime = time.time()
	main_ctrl(args)
	logging.info("Finished in %0.2f seconds."%(time.time() - starttime))

USAGE="""\
	Evaluate SV callset generated by simulations.
	Author: Shuqi Cao
	Email: sqcao@stu.hit.edu.cn
"""

def parseArgs(argv):
	parser = argparse.ArgumentParser(prog="evaluation on BND", description=USAGE, formatter_class=argparse.RawDescriptionHelpFormatter)
	parser.add_argument("base", type=str, help="Ground truth of BNDs.")
	parser.add_argument("comp", type=str, help="BND callsets to be benched.")
	parser.add_argument('-o', '--offect', help = "Offect of translocation overlaping.[%(default)s]", default = 1000, type = int)
	args = parser.parse_args(argv)
	return args

def setupLogging(debug=False):
	logLevel = logging.DEBUG if debug else logging.INFO
	logFormat = "%(asctime)s [%(levelname)s] %(message)s"
	logging.basicConfig( stream=sys.stderr, level=logLevel, format=logFormat )
	logging.info("Running %s" % " ".join(sys.argv))

if __name__ == '__main__':
	main(sys.argv[1:])